use std::{collections::HashMap, str::FromStr};

use serde::{Deserialize, Serialize};

pub mod bin;
pub mod build;
pub mod fs;
pub mod package;
pub mod ext;

#[derive(Debug, Clone, Serialize, Deserialize)]
#[serde(default)]
pub struct Manifest<E: Clone = Option<ext::Extension>> {
    pub package: package::Package,
    pub dependencies: HashMap<String, String>,
    pub fs: fs::FS,
    pub bin: Option<bin::Bin>,
    pub build: Option<build::Build>,
    pub ext: E,
}

impl<P: Clone> Manifest<P> {
    pub fn valid(&self) -> bool {
        if self.bin.is_none() && self.build.is_none() {
            return false;
        }
        true
    }
}

impl TryFrom<String> for Manifest {
    type Error = toml::de::Error;
    fn try_from(s: String) -> Result<Self, Self::Error> {
        toml::from_str(&s)
    }
}

impl FromStr for Manifest {
    type Err = toml::de::Error;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        toml::from_str(s)
    }
}

impl Default for Manifest {
    fn default() -> Manifest {
        Manifest {
            package: package::Package::default(),
            dependencies: HashMap::default(),
            fs: fs::FS::default(),
            bin: None,
            build: None,
            ext: None,
        }
    }
}
